@@ -14,13 +14,14 @@ import tempfile
 
 from contextlib import redirect_stdout
 from unittest.mock import patch
+from urllib.parse import ParseResult
 
 import pytest
 
 from courlan import cli
 from courlan import clean_url, normalize_url, scrub_url, check_url, is_external, sample_urls, validate_url, extract_links, extract_domain, fix_relative_urls, get_base_url, get_host_and_path, get_hostinfo, is_navigation_page, is_not_crawlable, lang_filter
 from courlan.filters import extension_filter, path_filter, spam_filter, type_filter
-from courlan.urlutils import is_known_link
+from courlan.urlutils import _parse, is_known_link
 
 
 logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
@@ -257,11 +258,22 @@ def test_urlcheck():
 
 def test_urlutils():
     '''Test URL manipulation tools'''
+    # domain extraction
     assert extract_domain('https://httpbin.org/') == 'httpbin.org'
+    # url parsing
+    result = _parse('https://httpbin.org/')
+    assert isinstance(result, ParseResult)
+    newresult = _parse(result)
+    assert isinstance(result, ParseResult)
+    with pytest.raises(TypeError):
+        result = _parse(float(1.23))
+
     assert get_base_url('https://example.org/path') == 'https://example.org'
     with pytest.raises(ValueError):
         assert get_host_and_path('123') is None
     assert get_host_and_path('https://example.org/path') == ('https://example.org', '/path')
+    assert get_host_and_path('https://example.org/') == ('https://example.org', '/')
+    assert get_host_and_path('https://example.org') == ('https://example.org', '/')
     assert get_hostinfo('https://httpbin.org/') == ('httpbin.org', 'https://httpbin.org')
     assert get_hostinfo('https://example.org/path') == ('example.org', 'https://example.org')
     # keeping track of known URLs
@@ -382,9 +394,22 @@ def test_sample():
 
     # assert len(sample_urls(['http://test.org/test1', 'http://test.org/test2'], 1)) == 1
     mylist = ['http://t.o/t1', 'http://test.org/test1', 'http://test.org/test2', 'http://test2.org/test2']
-    assert len(list(sample_urls(mylist, 1, verbose=True))) == 1
-    assert not list(sample_urls(mylist, 1, exclude_min=10, verbose=True))
-    assert not list(sample_urls(mylist, 1, exclude_max=1, verbose=True))
+    assert len(sample_urls(mylist, 1, verbose=True)) == 2
+    assert not sample_urls(mylist, 1, exclude_min=10, verbose=True)
+    assert len(sample_urls(mylist, 1, exclude_max=1, verbose=True)) == 1
+
+    test_urls = [f'https://test.org/{str(a)}' for a in range(1000)]
+    example_urls = [f'https://www.example.org/{str(a)}' for a in range(100)]
+    other_urls = [f'https://www.other.org/{str(a)}' for a in range(10000)]
+    urls = test_urls + example_urls + other_urls
+    sample = sample_urls(urls, 10)
+    assert len([u for u in sample if 'test.org' in u]) == 10
+    assert len([u for u in sample if 'example.org' in u]) == 10
+    assert len([u for u in sample if 'other.org' in u]) == 10
+    sample = sample_urls(urls, 150)
+    assert len([u for u in sample if 'test.org' in u]) == 150
+    assert len([u for u in sample if 'example.org' in u]) == 100
+    assert len([u for u in sample if 'other.org' in u]) == 150
 
 
 def test_examples():
